
function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}


Ext.define('DSS.addon.TimedEvents', {

    extend: 'DSS.addon.Framework',

    config: {
        appName: 'te_title',
        appId: 'system-addon-timed-events',
        appVersion: '1.2.0',
        appIcon: 'images/app_icon.png'
    },

    getHelp: function() {
        return Ext.create('Ext.Component', {
            loader: {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad: true,
                disableCaching: false
            }
        });
    },

    getContent: function() {
        
        var me=this;
        /* Parameter:
         *   * editor config
         *   * button config
         *   * store config   
         *   * columns
         *   * defaultSortCol
         * */
        
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                storeConfig: {
                    storeId : 'timedeventStore',
                    fields : [ {name:'id'}, 
                               {name:'disabled', sortType:'asBoolean'}, 
                               {name:'name', sortType:'asUCText'}, 
                               {name:'actions', sortType:'asAction'}, 
                               {name:'time', sortType:'asTime'}, 
                               {name:'recurrence', sortType:'asRecurrence'} ]  
                },
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                }, {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn', 
                    width : 200
                }, {
                    header : _('Activity'),
                    dataIndex : 'actions', 
                    maintainFlex : true,
                    flex : 5,
                    xtype: 'actioncolumn'
                }, {
                    header: _('Time'),
                    dataIndex: 'time',
                    xtype: 'timecolumn', 
                    width: 200
                }, {
                    header: _('Recurrence'),
                    dataIndex: 'recurrence',
                    xtype: 'recurrencecolumn', 
                    width: 160
                }],     
                buttonText: {
                    newButton: _('New event'),
                    editButton: _('Edit event'),
                    enableButton:_('Enable event'),
                    disableButton:_('Disable event'),
                    testButton: _('Test event'),
                    deleteButton: _('Delete event'),
                },
                editorConfig:{
                    title : _('Timed Event'),
                    staticControls : [ {
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type: 'hidden',
                        label: 'id',
                        id: 'id'
                    }],
                    tabControls : [{
                        tabLabel : _('time of execution'),
                        tabDescription : _('Choose the time, when the activities should be executed.'),
                        content : [{
                                label: _('Time'),
                                type : 'timeReference',
                                id : 'time'
                           },{
                               type : 'weekdaySelection',
                               id : 'recurrence',
                               label: _('Recurrence'),
                               flex : true
                          }]
                    }, {
                        tabLabel : _('Activity'),
                        tabDescription : _('Choose the activities, which should be executed.'),
                        content : [{
                            type : 'ActionConfigurator',
                            id : 'actions',
                            flex : true
                        }]
                    }]
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.loadParameter(grid);
                    },
                    'saveEntry':function (grid,values){                        
                        dss.raiseEvent('system-addon-timed-events.config', {
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        })
                        me.waitAndReload(grid);
                    },
                    'enableEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-timed-events.config', {
                            actions : 'enable',
                            id : entry.get('id')
                        });
                        me.waitAndReload(grid);
                     },
                    'disableEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-timed-events.config', {
                            actions : 'disable',
                            id : entry.get('id')
                        })
                        me.waitAndReload(grid);
                    },
                    'deleteEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-timed-events.config', {
                            actions : 'delete',
                            id : entry.get('id')
                        })
                        me.waitAndReload(grid);
                    },
                    'testEntry':function (grid,entry){
                        dss.ajaxAssyncRequestWithoutAnswer("/json/event/raise",
                                {name:'action_execute', parameter:  "path=/scripts/system-addon-timed-events/entries/" + entry.get('id')});                       
                    }
                }
        };
                      
        
        if (getParameterByName('mode')=='expert') {
            oConfig.editorConfig.tabControls[1].content[0].withDelay=true;
            oConfig.editorConfig.tabControls[0].content[0].randomize=1800;
        }
        return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid) {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'system-addon-timed-events.saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter(grid);
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'system-addon-timed-events.saved'
            });
        });
    },
    loadParameter : function(grid) {
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/system-addon-timed-events/entries/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        var oTimedEvent=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                             {name:'id',type:'simple'},
                                                             {name:'random',type:'simple'},
                                                             {name:'disabled',type:'condition/enabled'},
                                                             {name:'holiday',type:'condition/state'},
                                                             {name:'actions',type:'action'},
                                                             {name:'time',type:'time'},  //time : timebase: 'absolute', 'daily', 'sunrise', 'sunset' | offset: seconds
                                                             {name:'recurrence',type:'recurrence'}]);  // recurrence:  0 : MO,TU,WE,TH,FR,SA,SO        
        if (oTimedEvent!=null) {
            var oResultObject=[];
            for (var iIndex=0;iIndex<oTimedEvent.length;iIndex++) {
                var fAdd=true;
                if (oTimedEvent[iIndex]['holiday']!=null)
                    if (oTimedEvent[iIndex]['holiday']=='on')
                        fAdd=false;
                if (fAdd)
                    oResultObject.push(oTimedEvent[iIndex]);
            }
            grid.loadParameter(oResultObject);
        }
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/timed-events.po']);
    var timedevents = Ext.create('DSS.addon.TimedEvents');
    timedevents.initPage();
});

